@@ -10,10 +10,8 @@
 
 #define MAX_NT_PWD_LEN 129
 
-typedef int (*command_fn)(char *, char *, char *);
+typedef int command_fn(char *pwddb, char *name, char *password);
 
-int command_add_user(char *pwddb, char *account, char *password);
-int command_update_user(char *pwddb, char *account, char *password);
-int command_del_user(char *pwddb, char *account, char *unused);
+command_fn command_add_user, command_update_user, command_delete_user;
 
 #endif /* __KSMBD_USER_ADMIN_H__ */
