@@ -31,9 +31,6 @@ enum share_hosts {
 	KSMBD_SHARE_HOSTS_MAX,
 };
 
-#define KSMBD_SHARE_DEFAULT_CREATE_MASK	0744
-#define KSMBD_SHARE_DEFAULT_DIRECTORY_MASK	0755
-
 #define KSMBD_SHARE_INVALID_UID		((__u16)-1)
 #define KSMBD_SHARE_INVALID_GID		((__u16)-1)
 
@@ -121,9 +118,21 @@ enum KSMBD_SHARE_CONF {
 	KSMBD_SHARE_CONF_MAX
 };
 
-extern char *KSMBD_SHARE_CONF[KSMBD_SHARE_CONF_MAX];
+extern const char *KSMBD_SHARE_CONF[KSMBD_SHARE_CONF_MAX];
+extern const char *KSMBD_SHARE_DEFCONF[KSMBD_SHARE_CONF_MAX];
 
-int shm_share_config(char *k, enum KSMBD_SHARE_CONF c);
+#define KSMBD_SHARE_CONF_IS_GLOBAL(c) \
+	((c) == KSMBD_SHARE_CONF_GUEST_ACCOUNT || \
+	 (c) == KSMBD_SHARE_CONF_MAX_CONNECTIONS)
+
+#define KSMBD_SHARE_CONF_IS_BROKEN(c) \
+	((c) == KSMBD_SHARE_CONF_ADMIN_USERS || \
+	 (c) == KSMBD_SHARE_CONF_HOSTS_ALLOW || \
+	 (c) == KSMBD_SHARE_CONF_HOSTS_DENY || \
+	 (c) == KSMBD_SHARE_CONF_FOLLOW_SYMLINKS)
+
+int shm_share_name(char *name, char *p);
+int shm_share_config(const char *k, enum KSMBD_SHARE_CONF c);
 
 static inline void set_share_flag(struct ksmbd_share *share, int flag)
 {
@@ -150,9 +159,9 @@ int shm_add_new_share(struct smbconf_group *group);
 void shm_remove_all_shares(void);
 
 void shm_destroy(void);
-guint shm_share_name_hash(gconstpointer name);
-gboolean shm_share_name_equal(gconstpointer lname, gconstpointer rname);
-int shm_init(void);
+unsigned int shm_share_name_hash(const char *name);
+int shm_share_name_equal(const char *lname, const char *rname);
+void shm_init(void);
 
 int shm_lookup_users_map(struct ksmbd_share *share,
 			  enum share_users map,
@@ -165,10 +174,8 @@ int shm_lookup_hosts_map(struct ksmbd_share *share,
 int shm_open_connection(struct ksmbd_share *share);
 int shm_close_connection(struct ksmbd_share *share);
 
-typedef void (*walk_shares)(gpointer key,
-			    gpointer value,
-			    gpointer user_data);
-void for_each_ksmbd_share(walk_shares cb, gpointer user_data);
+typedef void (*share_cb)(struct ksmbd_share *share, void *data);
+void shm_iter_shares(share_cb cb, void *data);
 
 struct ksmbd_share_config_response;
 
