@@ -159,7 +159,7 @@ static void __rpc_pipe_free(struct ksmbd_rpc_pipe *pipe)
 	if (pipe->dce)
 		dcerpc_free(pipe->dce);
 	if (pipe->entries)
-		g_array_free(pipe->entries, 1);
+		g_ptr_array_free(pipe->entries, 1);
 	g_free(pipe);
 }
 
@@ -183,11 +183,7 @@ static struct ksmbd_rpc_pipe *rpc_pipe_alloc(void)
 		return NULL;
 
 	pipe->id = -1;
-	pipe->entries = g_array_new(0, 0, sizeof(void *));
-	if (!pipe->entries) {
-		rpc_pipe_free(pipe);
-		return NULL;
-	}
+	pipe->entries = g_ptr_array_new();
 	return pipe;
 }
 
@@ -212,15 +208,14 @@ static struct ksmbd_rpc_pipe *rpc_pipe_alloc_bind(unsigned int id)
 	return pipe;
 }
 
-static void free_hash_entry(gpointer k, gpointer s, gpointer user_data)
-{
-	__rpc_pipe_free(s);
-}
-
 static void __clear_pipes_table(void)
 {
+	struct ksmbd_rpc_pipe *pipe;
+	GHashTableIter iter;
+
 	g_rw_lock_writer_lock(&pipes_table_lock);
-	g_hash_table_foreach(pipes_table, free_hash_entry, NULL);
+	ghash_for_each(pipe, pipes_table, iter)
+		__rpc_pipe_free(pipe);
 	g_rw_lock_writer_unlock(&pipes_table_lock);
 }
 
@@ -249,7 +244,7 @@ static int try_realloc_payload(struct ksmbd_dcerpc *dce, size_t data_sz)
 		return -ENOMEM;
 	}
 
-	n = realloc(dce->payload, dce->payload_sz + 4096);
+	n = g_try_realloc(dce->payload, dce->payload_sz + 4096);
 	if (!n)
 		return -ENOMEM;
 
@@ -541,7 +536,7 @@ char *ndr_read_vstring(struct ksmbd_dcerpc *dce)
 		charset = KSMBD_CHARSET_UTF8;
 
 	if (raw_len == 0) {
-		out = strdup("");
+		out = g_strdup("");
 		return out;
 	}
 
@@ -588,14 +583,14 @@ int ndr_read_uniq_vstring_ptr(struct ksmbd_dcerpc *dce,
 
 void ndr_free_vstring_ptr(struct ndr_char_ptr *ctr)
 {
-	free(ctr->ptr);
+	g_free(ctr->ptr);
 	ctr->ptr = NULL;
 }
 
 void ndr_free_uniq_vstring_ptr(struct ndr_uniq_char_ptr *ctr)
 {
 	ctr->ref_id = 0;
-	free(ctr->ptr);
+	g_free(ctr->ptr);
 	ctr->ptr = NULL;
 }
 
@@ -633,9 +628,9 @@ static int __max_entries(struct ksmbd_dcerpc *dce, struct ksmbd_rpc_pipe *pipe)
 
 	current_size = 0;
 	for (i = 0; i < pipe->num_entries; i++) {
-		gpointer entry;
+		void *entry;
 
-		entry = g_array_index(pipe->entries,  gpointer, i);
+		entry = g_ptr_array_index(pipe->entries, i);
 		current_size += dce->entry_size(dce, entry);
 
 		if (current_size < 4 * dce->payload_sz / 5)
@@ -652,17 +647,17 @@ int __ndr_write_array_of_structs(struct ksmbd_rpc_pipe *pipe, int max_entry_nr)
 	int i;
 
 	for (i = 0; i < max_entry_nr; i++) {
-		gpointer entry;
+		void *entry;
 
-		entry = g_array_index(pipe->entries,  gpointer, i);
+		entry = g_ptr_array_index(pipe->entries, i);
 		if (dce->entry_rep(dce, entry))
 			return KSMBD_RPC_EBAD_DATA;
 	}
 
 	for (i = 0; i < max_entry_nr; i++) {
-		gpointer entry;
+		void *entry;
 
-		entry = g_array_index(pipe->entries,  gpointer, i);
+		entry = g_ptr_array_index(pipe->entries, i);
 		if (dce->entry_data(dce, entry))
 			return KSMBD_RPC_EBAD_DATA;
 	}
@@ -737,29 +732,25 @@ int ndr_write_array_of_structs(struct ksmbd_rpc_pipe *pipe)
 	return __ndr_write_array_of_structs(pipe, max_entry_nr);
 }
 
-int rpc_init(void)
+void rpc_init(void)
 {
-	pipes_table = g_hash_table_new(g_int_hash, g_int_equal);
 	if (!pipes_table)
-		return -ENOMEM;
-	g_rw_lock_init(&pipes_table_lock);
-	if (rpc_samr_init())
-		return -ENOMEM;
-	if (rpc_lsarpc_init())
-		return -ENOMEM;
-	return 0;
+		pipes_table = g_hash_table_new(g_int_hash, g_int_equal);
+
+	rpc_samr_init();
+	rpc_lsarpc_init();
 }
 
 void rpc_destroy(void)
 {
+	rpc_lsarpc_destroy();
+	rpc_samr_destroy();
+
 	if (pipes_table) {
 		__clear_pipes_table();
 		g_hash_table_destroy(pipes_table);
 		pipes_table = NULL;
 	}
-	g_rw_lock_clear(&pipes_table_lock);
-	rpc_samr_destroy();
-	rpc_lsarpc_destroy();
 }
 
 static int dcerpc_hdr_write(struct ksmbd_dcerpc *dce,
