@@ -58,7 +58,8 @@ static int __share_entry_size_ctr1(struct ksmbd_dcerpc *dce, gpointer entry)
 	int sz = 0;
 
 	sz = strlen(share->name) * 2;
-	sz += strlen(share->comment) * 2;
+	if (share->comment)
+		sz += strlen(share->comment) * 2;
 	sz += 9 * sizeof(__u32);
 	return sz;
 }
@@ -140,8 +141,7 @@ static int __share_entry_processed(struct ksmbd_rpc_pipe *pipe, int i)
 {
 	struct ksmbd_share *share;
 
-	share = g_array_index(pipe->entries,  gpointer, i);
-	pipe->entries = g_array_remove_index(pipe->entries, i);
+	share = g_ptr_array_remove_index(pipe->entries, i);
 	pipe->num_entries--;
 	pipe->num_processed++;
 	put_ksmbd_share(share);
@@ -149,11 +149,9 @@ static int __share_entry_processed(struct ksmbd_rpc_pipe *pipe, int i)
 	return 0;
 }
 
-static void __enum_all_shares(gpointer key, gpointer value, gpointer user_data)
+static void __enum_all_shares(struct ksmbd_share *share,
+			      struct ksmbd_rpc_pipe *pipe)
 {
-	struct ksmbd_rpc_pipe *pipe = (struct ksmbd_rpc_pipe *)user_data;
-	struct ksmbd_share *share = (struct ksmbd_share *)value;
-
 	if (!get_ksmbd_share(share))
 		return;
 
@@ -167,13 +165,13 @@ static void __enum_all_shares(gpointer key, gpointer value, gpointer user_data)
 		return;
 	}
 
-	pipe->entries = g_array_append_val(pipe->entries, share);
+	g_ptr_array_add(pipe->entries, share);
 	pipe->num_entries++;
 }
 
 static int srvsvc_share_enum_all_invoke(struct ksmbd_rpc_pipe *pipe)
 {
-	for_each_ksmbd_share(__enum_all_shares, pipe);
+	shm_iter_shares((share_cb)__enum_all_shares, pipe);
 	pipe->entry_processed = __share_entry_processed;
 	return 0;
 }
@@ -210,7 +208,7 @@ static int srvsvc_share_get_info_invoke(struct ksmbd_rpc_pipe *pipe,
 		}
 	}
 
-	pipe->entries = g_array_append_val(pipe->entries, share);
+	g_ptr_array_add(pipe->entries, share);
 	pipe->num_entries++;
 	pipe->entry_processed = __share_entry_processed;
 	return 0;
